package com.anoopknr.pastebin;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ShareActionProvider;
import android.widget.Spinner;

import java.io.BufferedWriter;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.security.SecureRandom;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;

public class MainActivity extends AppCompatActivity {

    private EditText poster, content;
    private Button pasteButton;
    private Spinner syntax;
    private String savedURL = "";
    private String POSTER = "guest";   // Default name given to poster is guest.
    private String SYNTAX = "text";    // Default type is set to plain text.
    private String EXPIRATION = "day"; // By default document expires in one day.
    private String CONTENT = "";
    private String PASTE_BIN_SITE = "https://paste.ubuntu.com/";
    private URL uri;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        pasteButton = (Button) findViewById(R.id.pasteButton);
        poster = (EditText) findViewById(R.id.poster);
        content = (EditText) findViewById(R.id.content);
        syntax = (Spinner) findViewById(R.id.syntax);

        ArrayAdapter<CharSequence> syntaxAdapter = ArrayAdapter.createFromResource(this, R.array.syntax_array, R.layout.simple_spinner_item);
        syntaxAdapter.setDropDownViewResource(R.layout.support_simple_spinner_dropdown_item);
        syntax.setAdapter(syntaxAdapter);

        pasteButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    POSTER = poster.getText().toString().trim();
                    CONTENT = content.getText().toString().trim();
                    SYNTAX = syntax.getSelectedItem().toString().trim();

                    if (POSTER.isEmpty() || CONTENT.isEmpty()) {
                        Snackbar.make(findViewById(R.id.pasteButton), R.string.null_input_error, Snackbar.LENGTH_SHORT).show();
                        return;
                    }
                    uri = new URL(PASTE_BIN_SITE);
                    savedURL = new GetResponse().execute(uri).get();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                if (savedURL.isEmpty()) {
                    Snackbar.make(findViewById(R.id.pasteButton), R.string.content_post_error, Snackbar.LENGTH_SHORT).show();
                    return;
                }
                // Alert Box
                AlertDialog.Builder pastebinAlert = new AlertDialog.Builder(MainActivity.this);
                pastebinAlert.setTitle("Saved To Pastebin").setMessage(savedURL).setPositiveButton(R.string.share, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {

                        Intent sharingIntent = new Intent(android.content.Intent.ACTION_SEND);
                        sharingIntent.setType("text/plain");
                        sharingIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, "Ubuntu Pastebox");
                        sharingIntent.putExtra(android.content.Intent.EXTRA_TEXT, savedURL );
                        startActivity(Intent.createChooser(sharingIntent, "Share via"));
                    }
                }).setNegativeButton(R.string.open, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        Intent intent = new Intent();
                        intent.setAction(Intent.ACTION_VIEW);
                        intent.addCategory(Intent.CATEGORY_BROWSABLE);
                        intent.setData(Uri.parse(savedURL));
                        startActivity(intent);
                    }
                });
                AlertDialog dialog = pastebinAlert.create();
                dialog.show();
                content.setText("");
                poster.setText("");
            }
        });

    }

    // function to encode body of POST method.
    public String getEncodedStrings() {
        String ENCODED_STRING = null;
        try {
            POSTER = URLEncoder.encode(POSTER, "UTF-8");
            SYNTAX = URLEncoder.encode(SYNTAX, "UTF-8");
            EXPIRATION = URLEncoder.encode(EXPIRATION, "UTF-8");
            CONTENT = URLEncoder.encode(CONTENT, "UTF-8");

            ENCODED_STRING = "poster=" + POSTER + "&syntax=" + SYNTAX + "&expiration=" + EXPIRATION + "&content=" + CONTENT;

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return ENCODED_STRING;
    }

    private class GetResponse extends AsyncTask<URL, Void, String> {

        @Override
        protected String doInBackground(URL... urls) {
            String savedURL = "";
            try {

                HttpsURLConnection connection = (HttpsURLConnection) urls[0].openConnection();

                SSLContext sc;
                sc = SSLContext.getInstance("TLS");
                sc.init(null, null, new SecureRandom());
                connection.setSSLSocketFactory(sc.getSocketFactory());

                connection.setReadTimeout(7000);      // Timeouts set to 7s.
                connection.setConnectTimeout(7000);
                connection.setRequestMethod("POST");
                connection.setDoOutput(true);
                connection.setDoInput(true);


                String StringToPost = getEncodedStrings();

                if (StringToPost != null) {
                    OutputStream outputStream = connection.getOutputStream();
                    BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
                    writer.write(StringToPost);
                    Log.i("PostBody", StringToPost);
                    writer.flush();
                    writer.close();
                    outputStream.close();
                }

                // Connection works only with this line.
                connection.getInputStream();

                savedURL = connection.getURL().toString();
                Log.i("New URL", savedURL);

                // Disconnecting
                connection.disconnect();

                if (savedURL.equals(PASTE_BIN_SITE)) // Site URL changes is POST was successful.
                {
                    savedURL = "";
                }
                Log.i("New URL", savedURL);

            } catch (Exception e) {
                if (e instanceof UnknownHostException) {
                    Snackbar.make(findViewById(R.id.pasteButton), R.string.host_error, Snackbar.LENGTH_SHORT).show();
                }
                e.printStackTrace();
            }
            return savedURL;
        }
    }
}
